# ANSYS Installer.
# Runs the ANSYS installer with preferred options, then adds firewall exceptions. Optionally deletes existing versions.
# Billy Constantine, 2017.
# 
# Extract your ANSYS installers to a folder (if you have multiple ISO files, extract them to the same folder), then copy this file into that folder.
# 
# Run with command line option "-aqwa" to use the AQWA settings.
# (There is a separate license server for AQWA and some additional products to install; by default, AQWA should not be installed.)
# Run with command line option "-nofw" to NOT apply firewall exception rules.
# Run with command line option "-delete" to remove existing ANSYS versions.
# 
# Should work for full versions and updates. Tested for 18.1, eyeballed and approved for 16.2 - 18.0, should work as far back as 14.0 (although it's only kitted out with settings back to 14.5).
# For later versions:
# - make sure to verify the Intel MPI version and update the setting of $intel_mpi accordingly (check common\WINX64.7z\commonfiles\MPI\Intel);
# - make sure to verify the Java Runtime Environment version and update the setting of $jre_version accordingly (check ekm\WINX64.7z\EKM\programs).

# Command line arguments "-aqwa", "-nofw", and "-delete", default to $false if not specified.
param([switch] $aqwa   = $false,
      [switch] $nofw   = $false,
      [switch] $delete = $false)

# The name of the software being installed
$software_name = "ANSYS"

# License server
$license_server = "ansys"

# TemplateInstallv001
$ScriptDir = Split-Path -Parent $MyInvocation.MyCommand.Path
Set-Location $ScriptDir

# Figure out the version of ANSYS being installed, and set up paths in the firewall exceptions accordingly.
# ANSYS install media includes an identifier file, to determine which "disc" is being installed. This is of
# the form "XXX-Y.dvd", where "XXX" is the version number without any dots and "Y" is the disc number. So we
# look for "XXX-1.dvd" which should be present in every update and every disc 1.
# Also check for InstallPreReqs.exe, which indicates if this is a full version or a patch.

$found_media = $false
$fileEntries = [IO.Directory]::GetFiles($ScriptDir)
foreach($fileName in $fileEntries) {
   $fileName = Split-Path $fileName -Leaf -Resolve
   if ($fileName -match "-1.dvd$") {
      $iversion = ([regex]::matches($fileName, "^\d{3}", "IgnoreCase"))
      $found_media = $true
   } elseif($fileName -ieq "InstallPreReqs.exe") {
      $full_version = $true
   }
}

if ($found_media -eq $false) {
   throw "Could not find media file ""XXX-1.dvd"" in ""$ScriptDir"""
}

# So we've found the version number with no dots (i.e. for 18.1 we've got "181") in $iversion.
# The variations we need are typically: v181, 18.1, 1810 and 18.1.0, so we'll build those.

[string] $inst_version = $iversion

$v_folder      = "v" + $inst_version
$version       = $inst_version.Insert(2, '.')
$fluent_folder = $version  + ".0"
$fluent_file   = $inst_version + "0"

$logfile = "$env:WinDir\Logs\$software_name$version" + "Install.log"
$logtext = Get-Date | Out-File $logfile -Append
$logtext = "Install started" | Out-File $logfile -Append
$logtext = "" | Out-File $logfile -Append

$logtext = "ANSYS install log: ""C:\Program Files\ANSYS Inc\install.log""" | Out-File $logfile -Append
$logtext = "ANSYS install error log: ""C:\Program Files\ANSYS Inc\install.err""" | Out-File $logfile -Append

if ($full_version -eq $true) {
   $logtext = "Installing full version" | Out-File $logfile -Append
} else {
   $logtext = "Installing update" | Out-File $logfile -Append
}

# Older versions should be run with "-disablerss".
# There are also different bundled MPI and JRE versions.

# With each new ANSYS release, the list of products for a silent install may change (e.g. version 18.0 and later dispensed with "acp" as a product).
# If the product list includes non-existent product codes, it will just install EVERYTHING.
# Consult the ANSYS Windows Installation Guide for product codes.

# This would look much nicer as a nested hash table, but I couldn't get it to work.

if ($version -eq "14.5") {
   
   $disablerss   = "-disablerss"
   $intel_mpi    = "4.0.3"
   $jre_version  = "1.6.0_17"
   $product_list = "-mechapdl -ansyscust -autodyn -lsdyna -cfdpost -cfx -turbogrid -fluent -polyflow -icemcfd -icemproe -icemug -icepak -catia4 -catia5 -parasolid -acis -solidworks -solidedge -adinventor -cocreate -jtopen -acad -proe -ug"

} elseif ($version -eq "15.0") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "4.1"
   $jre_version  = "1.7.0_07"
   $product_list = "-mechapdl -ansyscust -autodyn -lsdyna -cfdpost -cfx -turbogrid -fluent -polyflow -icemcfd -icemproe -icemug -icepak -proe -ug -catia4 -catia5 -catia6 -parasolid -acis -solidworks -solidedge -adinventor -cocreate -jtopen -acad"

} elseif ($version -eq "16.0") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "4.1.3"
   $jre_version  = "1.7.0_60"
   $product_list = "-acad -acis -acp -adinventor -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5 -catia5_reader -catia6 -cfx -cocreate -fluent -icemcfd -icepak -jtopen -lsdyna -mechapdl -parasolid -polyflow -proe -proe_reader -solidedge -solidworks -solidworks_reader -turbogrid -ug -ug_reader"

} elseif ($version -eq "16.1") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "4.1.3"
   $jre_version  = "1.7.0_60"
   $product_list = "-acad -acis -acp -adinventor -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5 -catia5_reader -catia6 -cfx -cocreate -fluent -icemcfd -icepak -jtopen -lsdyna -mechapdl -parasolid -polyflow -proe -proe_reader -solidedge -solidworks -solidworks_reader -turbogrid -ug -ug_reader"

} elseif ($version -eq "16.2") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "4.1.3"
   $jre_version  = "1.7.0_60"
   $product_list = "-acad -acis -acp -adinventor -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5 -catia5_reader -catia6 -cfx -cocreate -fluent -icemcfd -icepak -jtopen -lsdyna -mechapdl -parasolid -polyflow -proe -proe_reader -solidedge -solidworks -solidworks_reader -turbogrid -ug -ug_reader"

} elseif ($version -eq "17.0") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "5.0.3.048"
   $jre_version  = "1.8.0_40"
   $product_list = "-acad -acis -acp -adinventor -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5 -catia5_reader -catia6 -cfx -cocreate -fluent -icemcfd -icepak -jtopen -lsdyna -mechapdl -parasolid -polyflow -proe -proe_reader -solidedge -solidworks -solidworks_reader -turbogrid -ug -ug_reader"

} elseif ($version -eq "17.1") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "5.0.3.048"
   $jre_version  = "1.8.0_40"
   $product_list = "-acad -acis -acp -adinventor -aim -ansyscust -autodyn -catia4 -catia5 -catia6 -cfx -cocreate -fluent -forte -icemcfd -icepak -jtopen -lsdyna -mechapdl -mfl -parasolid -polyflow -proe_reader -solidedge -solidworks -turbogrid -ug_reader"

} elseif ($version -eq "17.2") {

   $disablerss   = "-disablerss"
   $intel_mpi    = "5.0.3.048"
   $jre_version  = "1.8.0_40"
   $product_list = "-acad -acis -acp -adinventor -aim -ansyscust -autodyn -catia4 -catia5 -catia6 -cfx -cocreate -fluent -forte -icemcfd -icepak -jtopen -lsdyna -mechapdl -mfl -parasolid -polyflow -proe_reader -solidedge -solidworks -turbogrid -ug_reader"

} elseif ($version -eq "18.0") {

   $disablerss   = ""
   $intel_mpi    = "5.1.3.180"
   $jre_version  = "1.8.0_91"
   $product_list = "-acad_reader -acis -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5_reader -catia6 -cfx -cocreate -fluent -forte -icemcfd -icepak -jtopen -lsdyna -mechapdl -chemkinpro -parasolid -polyflow -proe_reader -solidedge_reader -solidworks_reader -turbogrid -ug_reader"

} elseif ($version -eq "18.1") {

   $disablerss   = ""
   $intel_mpi    = "5.1.3.180"
   $jre_version  = "1.8.0_121"
   $product_list = "-acad_reader -acis -adinventor_reader -aim -ansyscust -autodyn -catia4 -catia5_reader -catia6 -cfx -cocreate -fluent -forte -icemcfd -icepak -jtopen -lsdyna -mechapdl -chemkinpro -parasolid -polyflow -proe_reader -solidedge_reader -solidworks_reader -turbogrid -ug_reader"

} else {

   $logtext = "ANSYS version $version is not supported by this installer. Exiting." | Out-File $logfile -Append
   exit

}

# If run with -aqwa, use the AQWA license server and product list (which is the same list but with "-aqwa" as well)
if ($aqwa -eq $true) {
   $license_server = "ansys-aqwa"
   $product_list   = "$product_list -aqwa"
   $logtext = "AQWA install requested" | Out-File $logfile -Append
}

$logtext = "Intel MPI: $intel_mpi" | Out-File $logfile -Append
$logtext = "EKM JRE version: $jre_version" | Out-File $logfile -Append
$logtext = "License Server: $license_server" | Out-File $logfile -Append
$logtext = "Product List: $product_list" | Out-File $logfile -Append

if ($delete -eq $true) {
   $logtext = "Deletion of existing versions requested" | Out-File $logfile -Append
   # ANSYS installation folder.
   $ansys_dir = "$env:ProgramFiles\ANSYS Inc"
   # Check if it exists
   if (Test-Path "$ansys_dir") {
      # Look for folders with names that start with 'v'.
      $installed_versions = (Get-ChildItem "$ansys_dir" | where { $_.Attributes -match 'Directory' -and $_.BaseName -match '^v'})
      if($installed_versions.Count -eq 0) {
         # No versions found!
         $logtext = "No versions found." | Out-File $logfile -Append
      } else {
         $logtext = "Versions found: $installed_versions" | Out-File $logfile -Append
         foreach ($v in $installed_versions) {
            # Check if the uninstaller (uninstall.exe) exists, and run it with '-silent'.
            # Run it within 'cmd.exe /c' to prevent multiple uninstalls running at the same time.
            $uninstaller = "$ansys_dir\$v\uninstall.exe"
            if (Test-Path "$uninstaller") {
               $uninstall = """$uninstaller"" -silent"
               $logtext = "Removing $v with command ""$uninstall""." | Out-File $logfile -Append
               cmd.exe /c $uninstall
            } else {
               # The uninstaller doesn't exist. 
               $logtext = "Cannot remove $version - $uninstaller does not exist. Remove manually." | Out-File $logfile -Append
            }
         }
      }
   } else {
      $logtext = "$ansys_dir does not exist, no action will be taken" | Out-File $logfile -Append
   }
}

# Build the install command.
# Full versions and patches both use "-silent" (and "-disablerss" in old versions). Only full versions have license server and product lists.
$install = "setup.exe -silent $disablerss"
if ($full_version -eq $true) {
   $install = $install + " -licserverinfo 2325:1055:$license_server $product_list"
}

# Run the ANSYS installer
$logtext = "Install Command: $install" | Out-File $logfile -Append
cmd.exe /c $install

# Add firewall exceptions

# A function to add a firewall exception for a program file, but only if the file exists
function AddFirewallException([string] $name, [string] $program) {
   if (Test-Path "$program") {
      netsh.exe advfirewall firewall add rule name="$name" dir=in action=allow program="$program" enable=yes protocol=TCP profile=domain
      netsh.exe advfirewall firewall add rule name="$name" dir=in action=allow program="$program" enable=yes protocol=UDP profile=domain
      $logtext = "Added firewall rule ""$name"" for ""$program""" | Out-File $logfile -Append
   } else {
      $logtext = "Ignoring firewall rule ""$name"" for ""$program"" (file not found)" | Out-File $logfile -Append
   }
}

# Apply exceptions only if it's a full installation, and it wasn't specifically requested to not with "-nofw"
if ($full_version -eq $true -and $nofw -eq $true) {
   
   $logtext = "Adding firewall exceptions" | Out-File $logfile -Append

   # Licensing
   AddFirewallException -name "ANSYS Licensing Interconnect Application.  Modified for ANSYS Release $version."  -program "C:\Program Files\ANSYS Inc\Shared Files\licensing\winx64\ansysli_client.exe"

   # Solvers
   AddFirewallException -name "AnsMeshingServer"                      -program "C:\Program Files\ANSYS Inc\$v_folder\aisol\bin\winx64\AnsMeshingServer.exe"
   AddFirewallException -name "AnsysWB Module"                        -program "C:\Program Files\ANSYS Inc\$v_folder\aisol\bin\winx64\ansyswbu.exe"
   AddFirewallException -name "Ansys.Services.SystemCoupling"         -program "C:\program files\ansys inc\$v_folder\aisol\bin\winx64\ansys.services.systemcoupling.exe"
   
   # Mechanical
   AddFirewallException -name "Mechanical APDL Program"               -program "C:\program files\ansys inc\$v_folder\ansys\bin\winx64\ansys.exe"
   
   # Remote Solve Manager
   AddFirewallException -name "ANSYS RSM Job Manager Service"         -program "C:\Program Files\ANSYS Inc\$v_folder\RSM\bin\ans.rsm.jmhost.exe"

   # Workbench
   AddFirewallException -name "ANSYS Workbench"                       -program "C:\Program Files\ANSYS Inc\$v_folder\Framework\bin\win64\ansysfww.exe"

   # Fluent
   AddFirewallException -name "cx$fluent_file"                        -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\cortex\win64\cx$fluent_file.exe"
   AddFirewallException -name "fl$fluent_file"                        -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\2d_host\fl$fluent_file.exe"
   AddFirewallException -name "fl$fluent_file"                        -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\2ddp_host\fl$fluent_file.exe"
   AddFirewallException -name "fl$fluent_file"                        -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\3d_host\fl$fluent_file.exe"
   AddFirewallException -name "fl$fluent_file"                        -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\3ddp_host\fl$fluent_file.exe"
   AddFirewallException -name "fl_mpi$fluent_file"                    -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\2d_node\fl_mpi$fluent_file.exe"
   AddFirewallException -name "fl_mpi$fluent_file"                    -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\2ddp_node\fl_mpi$fluent_file.exe"
   AddFirewallException -name "fl_mpi$fluent_file"                    -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\3d_node\fl_mpi$fluent_file.exe"
   AddFirewallException -name "fl_mpi$fluent_file"                    -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\win64\3ddp_node\fl_mpi$fluent_file.exe"
   AddFirewallException -name "mpid"                                  -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\multiport\mpi\win64\pcmpi\bin\mpid.exe"
   AddFirewallException -name "mpid"                                  -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\multiport\mpi\win64\ibmmpi\bin\mpid.exe"
   AddFirewallException -name "mpirun"                                -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\multiport\mpi\win64\pcmpi\bin\mpirun.exe"
   AddFirewallException -name "mpirun"                                -program "C:\Program Files\ANSYS Inc\$v_folder\fluent\fluent$fluent_folder\multiport\mpi\win64\ibmmpi\bin\mpirun.exe"

   # JRE
   AddFirewallException -name "Java(TM Platform SE binary)"           -program "C:\Program Files\ANSYS Inc\$v_folder\EKM\programs\jre$jre_version\bin\javaw.exe"

   # CFX
   AddFirewallException -name "PostEngine"                            -program "C:\Program Files\ANSYS Inc\$v_folder\CFX\bin\winnt-amd64\postengine.exe"
   AddFirewallException -name "PostGui_ogl"                           -program "C:\Program Files\ANSYS Inc\$v_folder\CFX\bin\winnt-amd64\postgui_ogl.exe"
   AddFirewallException -name "PreEngine"                             -program "C:\Program Files\ANSYS Inc\$v_folder\CFX\bin\winnt-amd64\preengine.exe"
   AddFirewallException -name "PreGui_ogl"                            -program "C:\Program Files\ANSYS Inc\$v_folder\CFX\bin\winnt-amd64\pregui_ogl.exe"
   AddFirewallException -name "viewer_ogl"                            -program "C:\Program Files\ANSYS Inc\$v_folder\CFX\bin\winnt-amd64\viewer_ogl.exe"

   # CFD Post
   AddFirewallException -name "viewer_ogl"                            -program "C:\Program Files\ANSYS Inc\$v_folder\CFD-Post\bin\winnt-amd64\viewer_ogl.exe"
   AddFirewallException -name "PostEngine"                            -program "C:\Program Files\ANSYS Inc\$v_folder\CFD-Post\bin\winnt-amd64\postengine.exe"
   AddFirewallException -name "PostGui_ogl"                           -program "C:\Program Files\ANSYS Inc\$v_folder\CFD-Post\bin\winnt-amd64\postgui_ogl.exe"

   # Spaceclaim
   AddFirewallException -name "SpaceClaim"                            -program "C:\Program Files\ANSYS Inc\$v_folder\scdm\spaceclaim.exe"

   # Turbogrid
   AddFirewallException -name "TGEngine"                              -program "C:\Program Files\ANSYS Inc\$v_folder\TurboGrid\bin\winnt-amd64\tgengine.exe"
   AddFirewallException -name "TGGui_ogl"                             -program "C:\Program Files\ANSYS Inc\$v_folder\TurboGrid\bin\winnt-amd64\tggui_ogl.exe"

   # TCL
   AddFirewallException -name "Wish Application"                      -program "C:\Program Files\ANSYS Inc\$v_folder\commonfiles\tcl\bin\winx64\wish.exe"

   # Help viewer
   AddFirewallException -name "ANSYS Help Viewer"                     -program "C:\program files\ansys inc\$v_folder\commonfiles\help\helpviewer\ansyshelpviewer.exe"

   # MPI
   AddFirewallException -name "PMI proxy for Hydra Process manager"   -program "C:\program files\ansys inc\$v_folder\commonfiles\MPI\Intel\$intel_mpi\winx64\bin\pmi_proxy.exe"
   AddFirewallException -name "Process manager for MPI applications"  -program "C:\program files\ansys inc\$v_folder\commonfiles\MPI\Intel\$intel_mpi\winx64\bin\mpiexec.exe"
}

$logtext = Get-Date | Out-File $logfile -Append
$logtext = "Install finished" | Out-File $logfile -Append
